<?php
namespace plugins\blog\controllers;

use \CRUDController;
use \IoC;
use plugins\blog\models\Post;
use plugins\antispam\library\InvisibleCaptcha;
use plugins\antispam\library\SimpleCaptcha;
use plugins\blog\models\Comment;
use \FilteredMap;

/**
 * Comment controller
 * @author Jérôme Brilland
 * @version 3.0
 * @copyright (c) 2022-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class CommentController extends CRUDController {

    protected $modelClassName = 'plugins\\blog\models\\Comment';

    public function doList() {
        parent::doList();
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.moderate_comments' );
        $i18n = IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'blog.comments' );
        $this->comments = $this->models;
    }

    public function doAdd() {
        $i18n = IoC::resolve( 'i18n' );
        
        $data = new FilteredMap( $this->request->getParameter( 'comment' ) );
        $post = Post::findOne( ['slug'], ['post_id' => $data->getInt( 'post_id' ), 'status' => Post::PUBLISHED, 'allow_comments' => 1] );
        if ( empty( $post ) ) {
            $this->forward( 'controllers\\ErrorController', 400 );
        }
        $comment = new Comment();
        $comment->post_id = $data->getInt( 'post_id' );
        $comment->name = $data->getString( 'name', null, 'unsafe_raw', FILTER_FLAG_STRIP_LOW );
        $comment->email = $data->getString( 'email' );
        $comment->body = $data->getString( 'body' );
        
        $this->request->setAttribute( 'id', $data->getInt( 'post_id' ) );
        IoC::resolve( 'view' )->assign( 'comment', $comment );
           
        //antispam
        //honeypot
        if ( !empty( $this->request->getParameter( 'remark' ) ) ) {
            IoC::resolve( 'logger' )->write( sprintf( '[%s] [%s] [%s] Potential spam detected for IP ' . $_SERVER['REMOTE_ADDR'], date( 'Y-m-d H:i:s' ), __FILE__, 'info' ) );
            die;
        }
        //form token
        $invisibleCaptcha = new InvisibleCaptcha();
        if ( !$invisibleCaptcha->checkToken( 'comment_form_' . $data->getInt( 'post_id' ), $this->request->getParameters()->getAlnum( 'token' ) ) ) {
            IoC::resolve( 'logger' )->write( sprintf( '[%s] [%s] [%s] Potential spam detected for IP ' . $_SERVER['REMOTE_ADDR'], date( 'Y-m-d H:i:s' ), __FILE__, 'info' ) );
            die;
        }
        //antispam answer
        $simpleCaptcha = new SimpleCaptcha();
        if ( !$simpleCaptcha->checkAnswer( $this->request->getParameters()->getInt( 'antispam_answer' ) ) ) {
            $this->flash->error( $i18n->__( 'wrong_antispam_answer' ) );
            $this->forward( 'plugins\\blog\\controllers\\PostController', 'show' );
        } else {
            
            if ( $comment->isInvalid() ) {
                $this->flash->error( array_shift( $comment->getErrors() )[0] );
                $this->forward( 'plugins\\blog\\controllers\\PostController', 'show' );
            } else {
                $comment->save();
                $this->flash->success( $i18n->__( 'blog.comment_successfully_posted' ) );
            }
        }
        $this->redirect( BASE_URL . 'blog/' . $post->slug . '.html#add-comment' );
    }

    public function doUpdateform() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.moderate_comments' );
        parent::doUpdateform();
    }

    public function doUpdate() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.moderate_comments' );
        parent::doUpdate();
    }

    public function doDelete() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.moderate_comments' );
        parent::doDelete();
    }

}
