<?php
namespace plugins\blog\controllers;

use \CRUDController;
use \IoC;
use plugins\blog\models\Post;
use models\Template;
use plugins\antispam\library\InvisibleCaptcha;
use plugins\antispam\library\SimpleCaptcha;

/**
 * Post controller
 * @author Jérôme Brilland
 * @version 3.0
 * @copyright (c) 2021-2023, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class PostController extends CRUDController {

    protected $modelClassName = 'plugins\\blog\\models\\Post';

    protected function beforeFilter() {
        $view = IoC::resolve( 'view' );
        $view->addTemplateDirectory( dirname( dirname( __FILE__ ) ) . DS . TEMPLATES_DIR );
    }

    public function doIndex() {
        $settings = IoC::resolve( 'settings' );
        $this->title = $settings['site_title'];
        $this->description = $settings['site_description'];
        
        $tag = $this->request->getAttribute( 'tag', null );
        $year = $this->request->getAttribute( 'year', null );
        
        $where = ['status' => Post::PUBLISHED,
                  'OR' => ['publication_date[<]' => date( 'Y-m-d' ),
                           'AND' => ['publication_date' => date( 'Y-m-d' ), 'publication_time[<=]' => date( 'H:i:s' )]],
                  ];

        $columns = [
            'post_id',
            'title',
            'language',
            'slug',
            'summary',
            'user_id',
            'allow_comments',
            'publication_date',
            'publication_time'
        ];

        if ( !empty( $tag ) ) {
            $join = ['[<]tagging' => ['post_id'=>'taggable_id'],
                     '[<]tag' => ['tagging.tag_id'=> 'tag_id']];
            $where['tag.name'] = $tag;
            $where['taggable_type'] = Post::getTableName();
        }

        if ( !empty( $year ) ) {
            $month = $this->request->getParameters()->getInt( 'month' );
            $start = new \DateTimeImmutable( $year . '-' . $month . '-01' );
            $end = $start->add( new \DateInterval( 'P1M') );
            $where['publication_date[>=]'] = $start->format( 'Y-m-d' );
            $where['publication_date[<]'] = $end->format( 'Y-m-d' );
        }

        if ( !empty( $tag ) ) {
            $totalItems = Post::count( $join, 'post_id', $where );
        } else {
            $totalItems = Post::count( $where );
        }
        
        $currentPage = $this->request->getAttribute( 'page', 1 );
        $paginator = new \Paginator( $settings['blog.posts_per_page'], $totalItems, $currentPage );
        $where['LIMIT'] = [$paginator->getOffset(), $paginator->getLimit()];

        $where['ORDER'] = ['publication_date' => 'DESC', 'publication_time' => 'DESC'];

        if ( !empty( $tag ) ) {
            $this->posts = Post::findMany( $join, $columns, $where );
        } else {
            $this->posts = Post::findMany( $columns, $where );
        }

        $nextPage = (integer)$currentPage + 1;
        if ( $paginator->isValidPage( $nextPage ) ) {
            if ( !empty( $tag ) ) {
                $this->olderPostsLink = BASE_URL . 'blog/tags/' . $tag . '/page/' . $nextPage;
            } else if ( !empty( $year ) ) {
                $this->olderPostsLink = BASE_URL . 'blog/'. $year . '/'. $month . '/page/' . $nextPage;
            } else {
                $this->olderPostsLink = BASE_URL . 'blog/page/' . $nextPage;
            }
        }
        $previousPage = (integer)$currentPage - 1;
        if ( $paginator->isValidPage( $previousPage ) ) {
            if ( !empty( $tag ) ) {
                $this->newerPostsLink = BASE_URL . 'blog/tags/' . $tag . '/page/' . $previousPage;
            } else if ( !empty( $year ) )  {
                $this->newerPostsLink = BASE_URL . 'blog/'. $year . '/'. $month . '/page/' . $previousPage;
            } else {
                $this->newerPostsLink = BASE_URL . 'blog/page/' . $previousPage;
            }
        }

        $this->user = IoC::resolve( 'user' );
        $this->tags = $this->getTags();

        $latte = IoC::resolve( 'view' )->getLatte();
        $latte->getLoader()->addLoader( IoC::resolve( 'latteDatabaseLoader') );

        $database = IoC::resolve( 'database' );
        $settings = IoC::resolve( 'settings' );
        $this->render( $database->get( 'template', ['name'], ['template_id' => $settings['blog.index_template'] ] )['name'] );

    }

    public function doList() {
        if ( 0 == Post::count() ) {
            $this->redirect( BASE_URL . 'blog/post/add' );
        }
        parent::doList();
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.view_blog_posts' );
        $i18n = IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'blog.posts' );
        $this->posts = $this->models;
    }

    public function doAddform() {
        parent::doAddform();
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.create_blog_post' );
        $i18n = IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'blog.add_post' );
    }

    public function doAdd() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.create_blog_post' );
        parent::doAdd();
    }

    public function doUpdateform() {
        parent::doUpdateform();
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.update_blog_post' );
        $i18n = IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'blog.update_post' );
        $this->template = 'post_updateform.html';
    }

    public function doUpdate() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.update_blog_post' );
        parent::doUpdate();
    }

    protected function afterAdd() {
        $i18n = IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'blog.post_successfully_created' );
        if ( $this->request->getParameters()->getBoolean( 'continue', false ) ) {
            $this->redirect( BASE_URL . 'blog/post/update/' . $this->model->post_id );
        } else {
            $this->redirect( BASE_URL . 'blog/post/list' );
        }
    }

    protected function afterUpdate() {
        $i18n = IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'blog.post_successfully_updated' );
        if ( $this->request->getParameters()->getBoolean( 'continue', false ) ) {
            $this->redirect( BASE_URL . 'blog/post/update/' . $this->model->post_id );
        } else {
            $this->redirect( BASE_URL . 'blog/post/list' );
        }
    }

    public function doDelete() {
        $authorization = IoC::resolve( 'authorization' );
        $authorization->authorize( 'blog.delete_blog_post' );
        parent::doDelete();
    }

    protected function afterDelete() {
        $i18n = IoC::resolve( 'i18n' );
        $this->flash['success'] = $i18n->__( 'blog.post_successfully_deleted' );
        $this->redirect( BASE_URL . 'blog/post/list' );
    }

    public function doShow() {
        $i18n = IoC::resolve( 'i18n' );
        
        $where = ['status' => Post::PUBLISHED,
                  'OR' => ['publication_date[<]' => date( 'Y-m-d' ),
                           'AND' => ['publication_date' => date( 'Y-m-d' ), 'publication_time[<=]' => date( 'H:i:s' )]],
        ];

        if ( !empty( $this->request->getAttribute( 'slug' ) ) ) {
            $where['slug'] = $this->request->getAttribute( 'slug' );
        } else {
            $where['post_id'] = $this->request->getAttribute( 'id' );
        }

        $columns = [
            'post_id',
            'title',
            'description',
            'language',
            'slug',
            'summary',
            'body',
            'user_id',
            'allow_comments',
            'publication_date',
            'publication_time'
        ];
        
        $this->post = Post::findOne( $columns, $where );
        if ( empty( $this->post ) ) {
            IoC::resolve( 'eventmediator' )->trigger( 'page_not_found', [$this, $this->request, $this->response] );
            $this->forward( 'controllers\\ErrorController', '404' );
        }

        $this->user = IoC::resolve( 'user' );
        $this->tags = $this->getTags();

        $latte = IoC::resolve( 'view' )->getLatte();
        $latte->getLoader()->addLoader( IoC::resolve( 'latteDatabaseLoader') );

        $database = IoC::resolve( 'database' );
        $settings = IoC::resolve( 'settings' );

        $this->title = $this->post->title . ' | ' . $settings['site_title'];

        //antispam
        if ( !class_exists( 'plugins\\antispam\\library\\InvisibleCaptcha' ) ) {
            throw new \RuntimeException( 'The Antispam plugin is missing.' );
        }

        if ( $this->post->allow_comments ) {
            $invisibleCaptcha = new InvisibleCaptcha();
            $this->token = $invisibleCaptcha->generateToken( 'comment_form_' . $this->post->post_id );

            $simpleCaptcha = new SimpleCaptcha();
            $this->question = $simpleCaptcha->generateQuestion();
        }

        $this->render( $database->get( 'template', ['name'], ['template_id' => $settings['blog.post_template'] ] )['name'] );
    }

    protected function getTags() {
        $database = IoC::resolve( 'database' );
        return $database->select( 'tag', ['[<]tagging' => 'tag_id'], ['tag.name', 'number' => \Medoo\Medoo::raw( 'COUNT(<name>)' )], ['taggable_type' => Post::getTableName(), 'GROUP' => 'name'] );
    }

}
